# coding: utf-8

from .base import BasePattern
from ..common.display import repr as _repr


class CyclePattern(BasePattern):
    """
    select an element regularly from the sequence.
    """
    def __init__(self, sequence, increment=1, interval=1, callback=None):
        """
        :param sequence: elements of sequence <list> or <dict> (element(key): element_count(value))
        :param increment: list-index increase. <int>
        :param interval: intervals up proceeds to the next element. <int>
        :param callback: function to be called at the end of rendering. <function>
        :return: Cycle Pattern instance.
        """
        assert interval > 0
        if isinstance(sequence, dict):
            self.sequence = []
            for value, num in sequence.items():
                self.sequence += [value] * num
        else:
            self.sequence = sequence
        self.size = len(self.sequence)
        self.index = 0
        self.count = 0
        self.interval = interval
        self.increment = increment
        self.add_callback(callback)

    def __repr__(self):
        return '<CyclePattern {0}>'.format(
            _repr(self.sequence, self.display_limit)
        )

    def render(self, *args):
        """
        render a element by regularly selection.
        :param args: any.
        :return: rendered element.
        """
        element = self.sequence[self.index % self.size]

        self.count += 1
        if self.count % self.interval == 0:
            self.index += self.increment
        for callback in self.callbacks:
            element = callback(element)
        return element

