# coding: utf-8

from .base import BasePattern
from ..common.display import ellipsis
import hashlib


class HashPattern(BasePattern):
    """
    generate hash value.
    """
    def __init__(self, subscript, method='md5', stringify=str, callback=None):
        """
        :param subscript: record subscript. <immutable>
        :param method: hash method type. <str> (md5 / sha1 / sha224 / sha256 / sha384 / sha512)
        :param stringify: stringify function to give hash method. <function>
        :param callback: function to be called at the end of rendering. <function>
        :return: Hash Pattern instance.
        """
        self.subscript = subscript
        self.method = method
        self.hash = hashlib.__dict__[method.lower()]
        self.stringify = stringify
        self.add_callback(callback)

    def __repr__(self):
        info = '{0} of record[{1}]'.format(
            self.method.upper(), self.subscript
        )
        return '<HashPattern {0}>'.format(
            ellipsis(info, self.display_limit)
        )

    def render(self, record, *args):
        """
        render a element by hash method.
        :param record: record to be written by pattern.render().
        :param args: any
        :return: rendered element.
        """
        seed = record[self.subscript]

        element = self.hash(self.stringify(seed)).hexdigest()
        for callback in self.callbacks:
            element = callback(element)
        return element
