# coding: utf-8

from .base import BasePattern
from ..common.display import repr as _repr
from ..exceptions import Break

from random import choice, randint


class PickoutPattern(BasePattern):
    """
    select an element randomly from the limited-choices.
    """
    def __init__(self, choices, missing=None, callback=None):
        """
        :param choices: elements of choices <list> or <dict> (element(key): element_count(value))
        :param missing: return this value if choices is blank. <any>
        :param callback: function to be called at the end of rendering. <function>
        :return: Choice Pattern instance.
        """
        if isinstance(choices, dict):
            self.choices = []
            for value, num in choices.items():
                self.choices += [value] * num
        else:
            self.choices = list(choices)
        self.missing = missing
        self.add_callback(callback)

    def __repr__(self):
        return '<PickoutPattern {0}>'.format(
            _repr(self.choices, self.display_limit)
        )

    def render(self, *args):
        """
        render a element by randomly selection
        :param args: any.
        :return: rendered element.
        """
        last_index = len(self.choices)
        if not last_index:
            return Break.apply(self.missing)

        pop_index = randint(0, last_index-1)
        element = self.choices.pop(pop_index)

        for callback in self.callbacks:
            element = callback(element)
        return element

